@@ -31,9 +31,11 @@
 #define AA_CLASS_MOUNT		7
 #define AA_CLASS_NS_DOMAIN	8
 #define AA_CLASS_PTRACE		9
+#define AA_CLASS_SIGNAL		10
 
-#define AA_CLASS_ENV		16
+#define AA_CLASS_LABEL		16
 
+/* defined in libapparmor's apparmor.h #define AA_CLASS_DBUS 32 */
 #define AA_CLASS_X		33
 
 #endif /* __AA_POLICYDB_H */
