@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/python3
 # ------------------------------------------------------------------
 #
 #    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
@@ -25,15 +25,6 @@ from apparmor.common import open_file_read, AppArmorException
 
 # XXX tests listed here will be *** SKIPPED *** XXX
 skip_startswith = (
-    # lots of invalid dbus rules (the tools currently just store them without any parsing)
-    'generated_dbus/bad-perms-',
-    'generated_dbus/bad-formatting-',
-    'generated_dbus/duplicated-conditionals-',
-    'generated_dbus/eavesdrop-incompat-',
-    'generated_dbus/message-incompat-',
-    'generated_dbus/pairing-unsupported-',
-    'generated_dbus/service-incompat-',
-
     # the tools don't check for conflicting x permissions (yet?)
     'generated_x/conflict-',
     'generated_x/ambiguous-',
@@ -56,31 +47,20 @@ exception_not_raised = [
     'capability/bad_3.sd',
     'capability/bad_4.sd',
     'change_hat/bad_parsing.sd',
-    'dbus/bad_bind_1.sd',
-    'dbus/bad_bind_2.sd',
-    'dbus/bad_eavesdrop_1.sd',
-    'dbus/bad_modifier_1.sd',
-    'dbus/bad_modifier_2.sd',
-    'dbus/bad_modifier_3.sd',
-    'dbus/bad_modifier_4.sd',
-    'dbus/bad_peer_1.sd',
+
+    # The tools don't detect conflicting change_profile exec modes
+    'change_profile/onx_conflict_unsafe1.sd',
+    'change_profile/onx_conflict_unsafe2.sd',
+
     'dbus/bad_regex_01.sd',
     'dbus/bad_regex_02.sd',
     'dbus/bad_regex_03.sd',
     'dbus/bad_regex_04.sd',
     'dbus/bad_regex_05.sd',
     'dbus/bad_regex_06.sd',
-    'file/bad_append_1.sd',
-    'file/bad_append_2.sd',
-    'file/bad_embedded_spaces_1.sd',
+    'file/bad_re_brace_1.sd',
     'file/bad_re_brace_2.sd',
     'file/bad_re_brace_3.sd',
-    'file/file/bad_append_1.sd',
-    'file/file/bad_embedded_spaces_1.sd',
-    'file/file/owner/bad_3.sd',
-    'file/file/owner/bad_5.sd',
-    'file/owner/bad_3.sd',
-    'file/owner/bad_5.sd',
     'mount/bad_opt_10.sd',
     'mount/bad_opt_11.sd',
     'mount/bad_opt_12.sd',
@@ -174,8 +154,6 @@ exception_not_raised = [
     'vars/vars_dbus_bad_01.sd',
     'vars/vars_dbus_bad_02.sd',
     'vars/vars_dbus_bad_03.sd',
-    'vars/vars_dbus_bad_04.sd',
-    'vars/vars_dbus_bad_05.sd',
     'vars/vars_dbus_bad_06.sd',
     'vars/vars_dbus_bad_07.sd',
     'vars/vars_file_evaluation_7.sd',
@@ -208,17 +186,10 @@ exception_not_raised = [
     'xtrans/simple_bad_conflicting_x_11.sd',
     'xtrans/simple_bad_conflicting_x_12.sd',
     'xtrans/simple_bad_conflicting_x_13.sd',
-    'xtrans/simple_bad_conflicting_x_14.sd',
-    'xtrans/simple_bad_conflicting_x_15.sd',
-    'xtrans/simple_bad_conflicting_x_1.sd',
     'xtrans/simple_bad_conflicting_x_2.sd',
-    'xtrans/simple_bad_conflicting_x_3.sd',
     'xtrans/simple_bad_conflicting_x_4.sd',
-    'xtrans/simple_bad_conflicting_x_5.sd',
     'xtrans/simple_bad_conflicting_x_6.sd',
-    'xtrans/simple_bad_conflicting_x_7.sd',
     'xtrans/simple_bad_conflicting_x_8.sd',
-    'xtrans/simple_bad_conflicting_x_9.sd',
     'xtrans/x-conflict.sd',
 ]
 
@@ -231,11 +202,9 @@ unknown_line = [
     'file/ok_other_2.sd',
     'file/ok_other_3.sd',
 
-    # permissions before path
+    # 'unsafe' keyword
     'file/file/front_perms_ok_1.sd',
     'file/front_perms_ok_1.sd',
-    'profile/local/local_named_profile_ok1.sd',
-    'profile/local/local_profile_ok1.sd',
     'xtrans/simple_ok_cx_1.sd',
 
     # permissions before path and owner / audit {...} blocks
@@ -287,10 +256,19 @@ syntax_failure = [
     'file/ok_5.sd',  # Invalid mode UX
     'file/ok_2.sd',  # Invalid mode RWM
     'file/ok_4.sd',  # Invalid mode iX
+    'file/ok_embedded_spaces_4.sd',  # \-escaped space
+    'file/file/ok_embedded_spaces_4.sd',  # \-escaped space
+    'file/ok_quoted_4.sd',  # quoted string including \"
     'xtrans/simple_ok_pix_1.sd',  # Invalid mode pIx
     'xtrans/simple_ok_pux_1.sd',  # Invalid mode rPux
 
+    # dbus regex mismatch
+    'vars/vars_dbus_4.sd',
+    'vars/vars_dbus_9.sd',
+    'vars/vars_dbus_2.sd',
+
     # misc
+    'vars/vars_dbus_8.sd',  # Path doesn't start with / or variable: {/@{TLDS}/foo,/com/@{DOMAINS}}
     'vars/vars_simple_assignment_12.sd',  # Redefining existing variable @{BAR} ('\' not handled)
     'rewrite/alias_good_5.sd',  # Values added to a non-existing variable @{FOO} (defined in include, lp:1331856)
     'bare_include_tests/ok_2.sd',  # two #include<...> in one line
