@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/python3
 # ------------------------------------------------------------------
 #
 #    Copyright (C) 2014 Canonical Ltd.
@@ -14,7 +14,8 @@ import unittest
 from common_test import AATest, setup_all_loops
 from apparmor.common import AppArmorBug, AppArmorException
 
-from apparmor.regex import strip_quotes, parse_profile_start_line, re_match_include, RE_PROFILE_START, RE_PROFILE_CAP, RE_PROFILE_PTRACE, RE_PROFILE_SIGNAL
+from apparmor.regex import ( strip_parenthesis, strip_quotes, parse_profile_start_line, re_match_include,
+     RE_PROFILE_START, RE_PROFILE_DBUS, RE_PROFILE_CAP, RE_PROFILE_PTRACE, RE_PROFILE_SIGNAL )
 
 
 class AARegexTest(AATest):
@@ -214,55 +215,17 @@ class AARegexCapability(AARegexTest):
         ('   capabilitynet_raw,', False)
     ]
 
-
-class AARegexPath(AARegexTest):
-    '''Tests for RE_PROFILE_PATH_ENTRY'''
-
-    def AASetup(self):
-        self.regex = aa.RE_PROFILE_PATH_ENTRY
-
-    tests = [
-        ('   /tmp/foo r,',
-         (None, None, None, None, '/tmp/foo', 'r', None, None, None)),
-        ('   audit /tmp/foo rw,',
-         ('audit', None, None, None, '/tmp/foo', 'rw', None, None, None)),
-        ('   audit deny /tmp/foo rw,',
-         ('audit', 'deny', None, None, '/tmp/foo', 'rw', None, None, None)),
-        ('   file /tmp/foo rw,',
-         (None, None, None, 'file', '/tmp/foo', 'rw', None, None, None)),
-        ('   file,', False),
-    ]
-
-
-class AARegexBareFile(AARegexTest):
-    '''Tests for RE_PROFILE_BARE_FILE_ENTRY'''
-
-    def AASetup(self):
-        self.regex = aa.RE_PROFILE_BARE_FILE_ENTRY
-
-    tests = [
-        ('   file,', (None, None, None, None)),
-        ('   dbus,', False),
-        ('   file /tmp/foo rw,', False),
-        ('   file /tmp/foo,', False),
-        ('   file r,', False),
-        ('  owner file  , ', (None, None, 'owner', None)),
-        ('  audit owner file  , ', ('audit', None, 'owner', None)),
-        ('  deny file  , ', (None, 'deny', None, None)),
-    ]
-
-
 class AARegexDbus(AARegexTest):
     '''Tests for RE_PROFILE_DBUS'''
 
     def AASetup(self):
-        self.regex = aa.RE_PROFILE_DBUS
+        self.regex = RE_PROFILE_DBUS
 
     tests = [
-        ('   dbus,', (None, None, 'dbus,', None)),
-        ('   audit dbus,', ('audit', None, 'dbus,', None)),
-        ('   dbus send member=no_comment,', (None, None, 'dbus send member=no_comment,', None)),
-        ('   dbus send member=no_comment, # comment', (None, None, 'dbus send member=no_comment,', '# comment')),
+        ('   dbus,',                                    (None,      None,   'dbus,',                            None,                       None)),
+        ('   audit dbus,',                              ('audit',   None,   'dbus,',                            None,                       None)),
+        ('   dbus send member=no_comment,',             (None,      None,   'dbus send member=no_comment,',     'send member=no_comment',   None)),
+        ('   dbus send member=no_comment, # comment',   (None,      None,   'dbus send member=no_comment,',     'send member=no_comment',   '# comment')),
 
         ('   dbusdriver,', False),
         ('   audit dbusdriver,', False),
@@ -501,6 +464,24 @@ class TestInvalid_re_match_include(AATest):
             re_match_include(params)
 
 
+class TestStripParenthesis(AATest):
+    tests = [
+        ('foo',         'foo'       ),
+        ('(foo)',       'foo'       ),
+        ('(  foo )',    'foo'       ),
+        ('(foo',        '(foo'      ),
+        ('foo  )',      'foo  )'    ),
+        ('foo ()',      'foo ()'    ),
+        ('()',          ''          ),
+        ('(  )',        ''          ),
+        ('(())',        '()'        ),
+        (' (foo)',       '(foo)'    ),  # parenthesis not first char, whitespace stripped nevertheless
+        ('(foo) ',       '(foo)'    ),  # parenthesis not last char, whitespace stripped nevertheless
+    ]
+
+    def _run_test(self, params, expected):
+        self.assertEqual(strip_parenthesis(params), expected)
+
 class TestStripQuotes(AATest):
     def test_strip_quotes_01(self):
         self.assertEqual('foo', strip_quotes('foo'))
